require_relative 'spec_helper'
require 'capybara'
require 'conformity/actions'


include Conformity
describe Conformity::Actions do
  before :each do
    @fields = double()
    @session = double()
    @actions = Actions.new(@fields, @session)
  end

  describe "#field" do
    it "get's the value of the field" do
      allow(@fields).to receive(:value).with(:name) { "Jane Doe" }
      expect(@actions.field(:name)).to match("Jane Doe")
    end
  end

  describe "actions" do
    before :each do
      @actions = Actions.new(@fields)
      @actions.visit "/form"
    end

    describe "#click_on" do
      it "clicks by text" do
        @actions.click_on "click me"
        expect(page).to include("click success")
      end

      it "clicks by css" do
        @actions.click_on "#click_me"
        expect(page).to include("click success")
      end
    end

    describe "#fill_in" do
      it "fills in by text" do
        @actions.fill_in "name", with: "Jane"
        @actions.click_on "Submit"
        expect(page).to include("Jane")
      end

      it "fills in by css" do
        @actions.fill_in "#name", with: "Jane"
        @actions.click_on "Submit"
        expect(page).to include("Jane")
      end
    end

    describe "#choose" do
      it "chooses by text" do
        @actions.choose "female"
        @actions.click_on "Submit"
        expect(page).to include("female")
      end

      it "chooses by css" do
        @actions.choose "#female"
        @actions.click_on "Submit"
        expect(page).to include("female")
      end
    end

    describe "#check" do
      it "checks by text" do
        @actions.check "chess"
        @actions.click_on "Submit"
        expect(page).to include("chess")
      end

      it "checks by css" do
        @actions.check "#chess"
        @actions.click_on "Submit"
        expect(page).to include("chess")
      end
    end

    describe "#uncheck" do
      it "unchecks by text" do
        @actions.uncheck "checkers"
        @actions.click_on "Submit"
        expect(page).to_not include("checkers")
      end

      it "unchecks by css" do
        @actions.uncheck "#checkers"
        @actions.click_on "Submit"
        expect(page).to_not include("checkers")
      end
    end

    describe "#select" do
      it "selects by text" do
        @actions.select("Mr")
        @actions.click_on "Submit"
        expect(page).to include("Mr")
      end

      it "selects by css" do
        @actions.select("#mr")
        @actions.click_on "Submit"
        expect(page).to include("Mr")
      end
    end

    describe "#unselect" do
      it "unselects by text" do
        @actions.unselect("BMW")
        @actions.click_on "Submit"
        expect(page).to_not include("BMW")
      end

      it "unselects by css" do
        @actions.unselect("option[value=bmw]")
        @actions.click_on "Submit"
        expect(page).to_not include("BMW")
      end
    end
  end

  describe "#method_missing" do
    it "delegates visit to session" do
      expect(@session).to receive(:visit).with("/")
      @actions.visit('/')
    end

    it "delegates find to session" do
      expect(@session).to receive(:find).with("#id")
      @actions.find("#id")
    end

    it "ignores success conditions" do
      expect { @actions.has_status_code?; @actions.has_content? }.to_not raise_error
    end
  end

  describe "logging" do
    it "calls session to save a screenshot" do
      Conformity.configure do |c|
        c.log_screenshots = true
        c.screenshots_folder = "."
      end

      allow(Time).to receive(:now) { 0 }
      allow(@session).to receive(:visit)
      expect(@session).to receive(:save_screenshot).with("./0_visit.png")

      @actions.visit('/')
    end
  end
end
